import re
import xml.etree.ElementTree as etree
from collections import defaultdict

from markdown import Markdown
from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor


class IconFontsPattern(InlineProcessor):
    """
    Return an <i> element with the necessary classes
    """

    _base: str
    _prefix: str

    def __init__(self, pattern: str, md: Markdown, base: str = "", prefix: str = ""):
        super().__init__(pattern, md)

        self._base = base
        self._prefix = prefix

    def handleMatch(self, m: re.Match, data: str):
        # The dictionary keys come from named capture groups in the regex
        match_dict = m.groupdict()

        el = etree.Element("i")

        name = match_dict["name"]

        classes: dict[str, str] = defaultdict(str)

        if self._base:
            classes["base"] = self._base

        classes["name"] = f"{self._prefix}{name}"

        # Mods are modifier classes. The syntax in the markdown is:
        # "&icon-namehere:2x;" and with multiple "&icon-spinner:2x,spin;"

        if match_dict["mod"] is not None:
            # Make a string with each modifier like: "fa-2x fa-spin"
            classes["mod"] = " ".join(
                map(
                    lambda c: self._prefix + c,
                    filter(None, match_dict["mod"].split(",")),
                )
            )

        # User mods are modifier classes that shouldn't be prefixed with
        # prefix. The syntax in the markdown is:
        # "&icon-namehere::red;" and with multiple "&icon-spinner::red,bold;"
        if match_dict["user_mod"] is not None:
            # Make a string with each modifier like "red bold"
            classes["user_mod"] = " ".join(
                filter(None, match_dict["user_mod"].split(","))
            )

        el.set("class", " ".join(classes.values()))

        # This is for accessibility and text-to-speech browsers
        # so they don't try to read it
        el.set("aria-hidden", "true")

        return el, m.start(0), m.end(0)


ICON_RE = r"(?P<name>[a-zA-Z0-9-]+)(:(?P<mod>[a-zA-Z0-9-]+(,[a-zA-Z0-9-]+)*)?(:(?P<user_mod>[a-zA-Z0-9-]+(,[a-zA-Z0-9-]+)*)?)?)?;"
#           ^---------------------^^ ^                    ^--------------^ ^ ^ ^                         ^--------------^ ^ ^ ^
#                                  | +-------------------------------------+ | +------------------------------------------+ | |
#                                  |                                         +----------------------------------------------+ |
#                                  +------------------------------------------------------------------------------------------+


class IconFontsExtension(Extension):
    """IconFonts Extension for Python-Markdown."""

    def __init__(self, **kwargs):

        self.config = {
            "prefix": ["icon-", "Custom class prefix."],
            "base": ["", "Base class added to each icon"],
            "prefix_base_pairs": [{}, "Prefix/base pairs"],
        }

        super().__init__(**kwargs)

    def extendMarkdown(self, md: Markdown):
        # Register the global pattern
        md.inlinePatterns.register(
            IconFontsPattern(
                f"&{self.getConfig('prefix')}{ICON_RE}",
                md,
                base=self.getConfig("base"),
                prefix=self.getConfig("prefix"),
            ),
            "iconfonts",
            175,
        )

        # Register each of the pairings
        for prefix, base in self.getConfig("prefix_base_pairs").items():
            md.inlinePatterns.register(
                IconFontsPattern(f"&{prefix}{ICON_RE}", md, base=base, prefix=prefix),
                f"iconfonts_{prefix.rstrip('-')}",
                175,
            )



import unittest

import markdown


class TestMDI(unittest.TestCase):
    def test_vanilla(self):
        text = "I love &icon-html5; and &icon-css3;."
        expected = '<p>I love <i aria-hidden="true" class="icon-html5"></i> and <i aria-hidden="true" class="icon-css3"></i>.</p>'

        md = markdown.Markdown(extensions=["iconfonts"])
        converted_text = md.convert(text)

        self.assertEqual(converted_text, expected)

    def test_vanilla_mod(self):
        text = "I also love &icon-spinner:spin; &icon-spinner:2x,spin;\n&icon-spinner:large,spin; Sorry we have to load..."
        expected = '<p>I also love <i aria-hidden="true" class="icon-spinner icon-spin"></i> <i aria-hidden="true" class="icon-spinner icon-2x icon-spin"></i>\n<i aria-hidden="true" class="icon-spinner icon-large icon-spin"></i> Sorry we have to load...</p>'

        md = markdown.Markdown(extensions=["iconfonts"])
        converted_text = md.convert(text)

        self.assertEqual(converted_text, expected)

    def test_vanilla_user_mod(self):
        text = "I also love &icon-spinner:spin:red; &icon-spinner:2x,spin:bold;\n&icon-spinner:large,spin; Sorry we have to load..."
        expected = '<p>I also love <i aria-hidden="true" class="icon-spinner icon-spin red"></i> <i aria-hidden="true" class="icon-spinner icon-2x icon-spin bold"></i>\n<i aria-hidden="true" class="icon-spinner icon-large icon-spin"></i> Sorry we have to load...</p>'

        md = markdown.Markdown(extensions=["iconfonts"])
        converted_text = md.convert(text)

        self.assertEqual(converted_text, expected)

    def test_prefix_base_pairs_setting(self):
        text = "I also love &fa-spinner:2x,spin:red; &glyphicon-remove::bold;\n&fa-spinner:large,spin; Sorry we have to load..."
        expected = '<p>I also love <i aria-hidden="true" class="fa fa-spinner fa-2x fa-spin red"></i> <i aria-hidden="true" class="glyphicon glyphicon-remove bold"></i>\n<i aria-hidden="true" class="fa fa-spinner fa-large fa-spin"></i> Sorry we have to load...</p>'

        md = markdown.Markdown(
            extensions=["iconfonts"],
            extension_configs={
                "iconfonts": {
                    "prefix": "icon-",
                    "base": "icon",
                    "prefix_base_pairs": {
                        "fa-": "fa",
                        "glyphicon-": "glyphicon",
                    },
                }
            },
        )
        converted_text = md.convert(text)

        self.assertEqual(converted_text, expected)

    def test_prefix_base_pairs_setting_with_normal_prefix_and_base_settings(self):
        text = "I also love &fa-spinner:2x,spin:red; &glyphicon-remove::bold;\n&icon-spinner:large,spin; Sorry we have to load..."
        expected = '<p>I also love <i aria-hidden="true" class="fa fa-spinner fa-2x fa-spin red"></i> <i aria-hidden="true" class="glyphicon glyphicon-remove bold"></i>\n<i aria-hidden="true" class="icon icon-spinner icon-large icon-spin"></i> Sorry we have to load...</p>'

        md = markdown.Markdown(
            extensions=["iconfonts"],
            extension_configs={
                "iconfonts": {
                    "prefix": "icon-",
                    "base": "icon",
                    "prefix_base_pairs": {
                        "fa-": "fa",
                        "glyphicon-": "glyphicon",
                    },
                }
            },
        )
        converted_text = md.convert(text)

        self.assertEqual(converted_text, expected)

    def test_custom_prefix(self):
        text = "I love &mypref-html5; and &mypref-css3;."
        expected = '<p>I love <i aria-hidden="true" class="mypref-html5"></i> and <i aria-hidden="true" class="mypref-css3"></i>.</p>'

        md = markdown.Markdown(
            extensions=["iconfonts"],
            extension_configs={"iconfonts": {"prefix": "mypref-"}},
        )
        converted_text = md.convert(text)

        self.assertEqual(converted_text, expected)

    def test_custom_prefix_mod(self):
        text = "I also love &mypref-spinner:spin; &mypref-spinner:2x,spin;\n&mypref-spinner:large,spin; Sorry we have to load..."
        expected = '<p>I also love <i aria-hidden="true" class="mypref-spinner mypref-spin"></i> <i aria-hidden="true" class="mypref-spinner mypref-2x mypref-spin"></i>\n<i aria-hidden="true" class="mypref-spinner mypref-large mypref-spin"></i> Sorry we have to load...</p>'

        md = markdown.Markdown(
            extensions=["iconfonts"],
            extension_configs={"iconfonts": {"prefix": "mypref-"}},
        )
        converted_text = md.convert(text)

        self.assertEqual(converted_text, expected)

    def test_base(self):
        text = "I love &icon-html5; and &icon-css3;."
        expected = '<p>I love <i aria-hidden="true" class="icon icon-html5"></i> and <i aria-hidden="true" class="icon icon-css3"></i>.</p>'

        md = markdown.Markdown(
            extensions=["iconfonts"],
            extension_configs={"iconfonts": {"base": "icon"}},
        )
        converted_text = md.convert(text)

        self.assertEqual(converted_text, expected)

    def test_complex(self):
        text = "I love &fa-html5; and &fa-css3;."
        expected = '<p>I love <i aria-hidden="true" class="fa fa-html5"></i> and <i aria-hidden="true" class="fa fa-css3"></i>.</p>'

        md = markdown.Markdown(
            extensions=["iconfonts"],
            extension_configs={"iconfonts": {"prefix": "fa-", "base": "fa"}},
        )
        converted_text = md.convert(text)

        self.assertEqual(converted_text, expected)
